@@ -1,12 +1,13 @@
 
-// Compiler implementation of the D programming language
-// Copyright (c) 1999-2013 by Digital Mars
-// All Rights Reserved
-// written by Walter Bright
-// http://www.digitalmars.com
-// License for redistribution is by either the Artistic License
-// in artistic.txt, or the GNU General Public License in gnu.txt.
-// See the included readme.txt for details.
+/* Compiler implementation of the D programming language
+ * Copyright (c) 1999-2014 by Digital Mars
+ * All Rights Reserved
+ * written by Walter Bright
+ * http://www.digitalmars.com
+ * Distributed under the Boost Software License, Version 1.0.
+ * http://www.boost.org/LICENSE_1_0.txt
+ * https://github.com/D-Programming-Language/dmd/blob/master/src/module.c
+ */
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -20,12 +21,10 @@
 #include "id.h"
 #include "import.h"
 #include "dsymbol.h"
-#include "hdrgen.h"
+#include "expression.h"
 #include "lexer.h"
-
-#ifdef IN_GCC
-#include "d-dmd-gcc.h"
-#endif
+#include "attrib.h"
+#include "target.h"
 
 AggregateDeclaration *Module::moduleinfo;
 
@@ -48,9 +47,6 @@ Module::Module(const char *filename, Identifier *ident, int doDocComment, int do
         : Package(ident)
 {
     const char *srcfilename;
-#if IN_DMD
-    const char *symfilename;
-#endif
 
 //    printf("Module::Module(filename = '%s', ident = '%s')\n", filename, ident->toChars());
     this->arg = filename;
@@ -59,9 +55,14 @@ Module::Module(const char *filename, Identifier *ident, int doDocComment, int do
     numlines = 0;
     members = NULL;
     isDocFile = 0;
+    isPackageFile = false;
     needmoduleinfo = 0;
     selfimports = 0;
+    rootimports = 0;
     insearch = 0;
+    searchCacheIdent = NULL;
+    searchCacheSymbol = NULL;
+    searchCacheFlags = 0;
     decldefs = NULL;
 #if IN_DMD
     massert = NULL;
@@ -91,7 +92,6 @@ Module::Module(const char *filename, Identifier *ident, int doDocComment, int do
     macrotable = NULL;
     escapetable = NULL;
 #if IN_DMD
-    safe = false;
     doppelganger = 0;
     cov = NULL;
     covb = NULL;
@@ -122,8 +122,6 @@ Module::Module(const char *filename, Identifier *ident, int doDocComment, int do
 #if IN_DMD
     objfile = setOutfile(global.params.objname, global.params.objdir, filename, global.obj_ext);
 
-    symfilename = FileName::forceExt(filename, global.sym_ext);
-
     if (doDocComment)
         setDocfile();
 
@@ -131,16 +129,16 @@ Module::Module(const char *filename, Identifier *ident, int doDocComment, int do
         hdrfile = setOutfile(global.params.hdrname, global.params.hdrdir, arg, global.hdr_ext);
 
     //objfile = new File(objfilename);
-    symfile = new File(symfilename);
 #endif
 #if IN_LLVM
     // LDC
     llvmForceLogging = false;
     noModuleInfo = false;
-    moduleInfoVar = NULL;
     this->doDocComment = doDocComment;
     this->doHdrGen = doHdrGen;
     this->arrayfuncs = 0;
+    d_cover_valid = NULL;
+    d_cover_data = NULL;
 #endif
 }
 
@@ -191,7 +189,8 @@ File *Module::setOutfile(const char *name, const char *dir, const char *arg, con
     }
 
     if (FileName::equals(docfilename, srcfile->name->str))
-    {   error("Source file and output file have same name '%s'", srcfile->name->str);
+    {
+        error("source file and output file have same name '%s'", srcfile->name->str);
         fatal();
     }
 
@@ -215,16 +214,14 @@ const char *Module::kind()
 }
 
 Module *Module::load(Loc loc, Identifiers *packages, Identifier *ident)
-{   Module *m;
-    char *filename;
-
+{
     //printf("Module::load(ident = '%s')\n", ident->toChars());
 
     // Build module filename by turning:
     //  foo.bar.baz
     // into:
     //  foo\bar\baz
-    filename = ident->toChars();
+    char *filename = ident->toChars();
     if (packages && packages->dim)
     {
         OutBuffer buf;
@@ -244,7 +241,7 @@ Module *Module::load(Loc loc, Identifiers *packages, Identifier *ident)
         filename = (char *)buf.extractData();
     }
 
-    m = new Module(filename, ident, 0, 0);
+    Module *m = new Module(filename, ident, 0, 0);
     m->loc = loc;
 
     /* Look for the source file
@@ -253,27 +250,26 @@ Module *Module::load(Loc loc, Identifiers *packages, Identifier *ident)
     if (result)
         m->srcfile = new File(result);
 
+    if (!m->read(loc))
+        return NULL;
+
     if (global.params.verbose)
     {
         fprintf(global.stdmsg, "import    ");
         if (packages)
         {
             for (size_t i = 0; i < packages->dim; i++)
-            {   Identifier *pid = (*packages)[i];
+            {
+                Identifier *pid = (*packages)[i];
                 fprintf(global.stdmsg, "%s.", pid->toChars());
             }
         }
         fprintf(global.stdmsg, "%s\t(%s)\n", ident->toChars(), m->srcfile->toChars());
     }
 
-    if (!m->read(loc))
-        return NULL;
+    m = m->parse();
 
-    m->parse();
-
-#ifdef IN_GCC
-    d_gcc_magic_module(m);
-#endif
+    Target::loadModule(m);
 
     return m;
 }
@@ -291,7 +287,7 @@ bool Module::read(Loc loc)
             errorSupplemental(loc, "Please check your ldc2.conf configuration file.");
             errorSupplemental(loc, "Installation instructions can be found at http://wiki.dlang.org/LDC.");
 #else
-            errorSupplemental(loc, "dmd might not be correctly installed. Run 'dmd -man' for installation instructions.");
+            errorSupplemental(loc, "config file: %s", FileName::canonicalName(global.inifilename));
 #endif
         }
         else
@@ -308,7 +304,8 @@ bool Module::read(Loc loc)
         }
 
         if (!global.gag)
-        {   /* Print path
+        {
+            /* Print path
              */
             if (global.path)
             {
@@ -327,89 +324,15 @@ bool Module::read(Loc loc)
     return true;
 }
 
-inline unsigned readwordLE(unsigned short *p)
+Module *Module::parse(bool gen_docs)
 {
-#if IN_LLVM
-#if __LITTLE_ENDIAN__
-    return *p;
-#else
-    return (((unsigned char *)p)[1] << 8) | ((unsigned char *)p)[0];
-#endif
-#else
-#if LITTLE_ENDIAN
-    return *p;
-#else
-    return (((unsigned char *)p)[1] << 8) | ((unsigned char *)p)[0];
-#endif
-#endif
-}
-
-inline unsigned readwordBE(unsigned short *p)
-{
-#if IN_LLVM
-#if __BIG_ENDIAN__
-    return *p;
-#else
-    return (((unsigned char *)p)[0] << 8) | ((unsigned char *)p)[1];
-#endif
-#else
-    return (((unsigned char *)p)[0] << 8) | ((unsigned char *)p)[1];
-#endif
-}
-
-inline unsigned readlongLE(unsigned *p)
-{
-#if IN_LLVM
-#if __LITTLE_ENDIAN__
-    return *p;
-#else
-    return ((unsigned char *)p)[0] |
-        (((unsigned char *)p)[1] << 8) |
-        (((unsigned char *)p)[2] << 16) |
-        (((unsigned char *)p)[3] << 24);
-#endif
-#else
-#if LITTLE_ENDIAN
-    return *p;
-#else
-    return ((unsigned char *)p)[0] |
-        (((unsigned char *)p)[1] << 8) |
-        (((unsigned char *)p)[2] << 16) |
-        (((unsigned char *)p)[3] << 24);
-#endif
-#endif
-}
-
-inline unsigned readlongBE(unsigned *p)
-{
-#if IN_LLVM
-#if __BIG_ENDIAN__
-    return *p;
-#else
-    return ((unsigned char *)p)[3] |
-        (((unsigned char *)p)[2] << 8) |
-        (((unsigned char *)p)[1] << 16) |
-        (((unsigned char *)p)[0] << 24);
-#endif
-#else
-    return ((unsigned char *)p)[3] |
-        (((unsigned char *)p)[2] << 8) |
-        (((unsigned char *)p)[1] << 16) |
-        (((unsigned char *)p)[0] << 24);
-#endif
-}
-
-#if IN_LLVM
-void Module::parse(bool gen_docs)
-#else
-void Module::parse()
-#endif
-{
-    //printf("Module::parse()\n");
+    //printf("Module::parse(srcfile='%s') this=%p\n", srcfile->name->toChars(), this);
 
     char *srcname = srcfile->name->toChars();
     //printf("Module::parse(srcname = '%s')\n", srcname);
 
+    isPackageFile = (strcmp(srcfile->name->name(), "package.d") == 0);
+
     utf8_t *buf = (utf8_t *)srcfile->buffer;
     size_t buflen = srcfile->len;
 
@@ -446,7 +369,7 @@ void Module::parse()
                 for (pu += bom; pu < pumax; pu++)
                 {   unsigned u;
 
-                    u = le ? readlongLE(pu) : readlongBE(pu);
+                    u = le ? Port::readlongLE(pu) : Port::readlongBE(pu);
                     if (u & ~0x7F)
                     {
                         if (u > 0x10FFFF)
@@ -481,7 +404,7 @@ void Module::parse()
                 for (pu += bom; pu < pumax; pu++)
                 {   unsigned u;
 
-                    u = le ? readwordLE(pu) : readwordBE(pu);
+                    u = le ? Port::readwordLE(pu) : Port::readwordBE(pu);
                     if (u & ~0x7F)
                     {   if (u >= 0xD800 && u <= 0xDBFF)
                         {   unsigned u2;
@@ -490,7 +413,7 @@ void Module::parse()
                             {   error("surrogate UTF-16 high value %04x at EOF", u);
                                 fatal();
                             }
-                            u2 = le ? readwordLE(pu) : readwordBE(pu);
+                            u2 = le ? Port::readwordLE(pu) : Port::readwordBE(pu);
                             if (u2 < 0xDC00 || u2 > 0xDFFF)
                             {   error("surrogate UTF-16 low value %04x out of range", u2);
                                 fatal();
@@ -587,7 +510,7 @@ void Module::parse()
         if (!docfile)
             setDocfile();
 #endif
-        return;
+        return this;
     }
     {
 #if IN_LLVM
@@ -599,6 +522,8 @@ void Module::parse()
         members = p.parseModule();
         md = p.md;
         numlines = p.scanloc.linnum;
+        if (p.errors)
+            ++global.errors;
     }
 
     if (srcfile->ref == 0)
@@ -617,7 +542,6 @@ void Module::parse()
          * the name of this module.
          */
         this->ident = md->id;
-        this->safe = md->safe;
         Package *ppack = NULL;
         dst = Package::resolve(md->packages, &this->parent, &ppack);
         assert(dst);
@@ -642,10 +566,72 @@ void Module::parse()
             error("has non-identifier characters in filename, use module declaration instead");
     }
 
+    // Add internal used functions in 'object' module members.
+    if (!parent && ident == Id::object)
+    {
+        static const utf8_t code_ArrayEq[] =
+            "bool _ArrayEq(T1, T2)(T1[] a, T2[] b) {\n"
+            " if (a.length != b.length) return false;\n"
+            " foreach (size_t i; 0 .. a.length) { if (a[i] != b[i]) return false; }\n"
+            " return true; }\n";
+
+        static const utf8_t code_ArrayPostblit[] =
+            "void _ArrayPostblit(T)(T[] a) { foreach (ref T e; a) e.__xpostblit(); }\n";
+
+        static const utf8_t code_ArrayDtor[] =
+            "void _ArrayDtor(T)(T[] a) { foreach_reverse (ref T e; a) e.__xdtor(); }\n";
+
+        static const utf8_t code_xopEquals[] =
+            "bool _xopEquals(in void*, in void*) { throw new Error(\"TypeInfo.equals is not implemented\"); }\n";
+
+        static const utf8_t code_xopCmp[] =
+            "bool _xopCmp(in void*, in void*) { throw new Error(\"TypeInfo.compare is not implemented\"); }\n";
+
+        Identifier *arreq = Id::_ArrayEq;
+        Identifier *xopeq = Identifier::idPool("_xopEquals");
+        Identifier *xopcmp = Identifier::idPool("_xopCmp");
+        for (size_t i = 0; i < members->dim; i++)
+        {
+            Dsymbol *sx = (*members)[i];
+            if (!sx) continue;
+            if (arreq && sx->ident == arreq) arreq = NULL;
+            if (xopeq && sx->ident == xopeq) xopeq = NULL;
+            if (xopcmp && sx->ident == xopcmp) xopcmp = NULL;
+        }
+
+        if (arreq)
+        {
+            Parser p(loc, this, code_ArrayEq, strlen((const char *)code_ArrayEq), 0);
+            p.nextToken();
+            members->append(p.parseDeclDefs(0));
+        }
+        {
+            Parser p(loc, this, code_ArrayPostblit, strlen((const char *)code_ArrayPostblit), 0);
+            p.nextToken();
+            members->append(p.parseDeclDefs(0));
+        }
+        {
+            Parser p(loc, this, code_ArrayDtor, strlen((const char *)code_ArrayDtor), 0);
+            p.nextToken();
+            members->append(p.parseDeclDefs(0));
+        }
+        if (xopeq)
+        {
+            Parser p(loc, this, code_xopEquals, strlen((const char *)code_xopEquals), 0);
+            p.nextToken();
+            members->append(p.parseDeclDefs(0));
+        }
+        if (xopcmp)
+        {
+            Parser p(loc, this, code_xopCmp, strlen((const char *)code_xopCmp), 0);
+            p.nextToken();
+            members->append(p.parseDeclDefs(0));
+        }
+    }
+
     // Insert module into the symbol table
     Dsymbol *s = this;
-    bool isPackageMod = strcmp(srcfile->name->name(), "package.d") == 0;
-    if (isPackageMod)
+    if (isPackageFile)
     {
         /* If the source tree is as follows:
          *     pkg/
@@ -680,16 +666,22 @@ void Module::parse()
         assert(prev);
         if (Module *mprev = prev->isModule())
         {
-            if (strcmp(srcname, mprev->srcfile->toChars()) == 0)
-                error(loc, "from file %s must be imported as module '%s'",
-                    srcname, toPrettyChars());
-            else
+            if (FileName::compare(srcname, mprev->srcfile->toChars()) != 0)
                 error(loc, "from file %s conflicts with another module %s from file %s",
                     srcname, mprev->toChars(), mprev->srcfile->toChars());
+            else if (isRoot() && mprev->isRoot())
+                error(loc, "from file %s is specified twice on the command line",
+                    srcname);
+            else
+                error(loc, "from file %s must be imported with 'import %s;'",
+                    srcname, toPrettyChars());
+
+            // Bugzilla 14446: Return previously parsed module to avoid AST duplication ICE.
+            return mprev;
         }
         else if (Package *pkg = prev->isPackage())
         {
-            if (pkg->isPkgMod == PKGunknown && isPackageMod)
+            if (pkg->isPkgMod == PKGunknown && isPackageFile)
             {
                 /* If the previous inserted Package is not yet determined as package.d,
                  * link it to the actual module.
@@ -698,7 +690,7 @@ void Module::parse()
                 pkg->mod = this;
             }
             else
-                error(pkg->loc, "from file %s conflicts with package name %s",
+                error(md ? md->loc : loc, "from file %s conflicts with package name %s",
                     srcname, pkg->toChars());
         }
         else
@@ -709,6 +701,7 @@ void Module::parse()
         // Add to global array of all modules
         amodules.push(this);
     }
+    return this;
 }
 
 void Module::importAll(Scope *prevsc)
@@ -724,6 +717,14 @@ void Module::importAll(Scope *prevsc)
         return;
     }
 
+    if (md && md->msg)
+    {
+        if (StringExp *se = md->msg->toStringExp())
+            md->msg = se;
+        else
+            md->msg->error("string expected, not '%s'", md->msg->toChars());
+    }
+
     /* Note that modules get their own scope, from scratch.
      * This is so regardless of where in the syntax a module
      * gets imported, it is unaffected by context.
@@ -748,7 +749,7 @@ void Module::importAll(Scope *prevsc)
         for (size_t i = 0; i < members->dim; i++)
         {
             Dsymbol *s = (*members)[i];
-            s->addMember(NULL, sc->scopesym, 1);
+            s->addMember(sc, sc->scopesym);
         }
     }
     // anything else should be run after addMember, so version/debug symbols are defined
@@ -794,34 +795,6 @@ void Module::semantic()
 
     //printf("Module = %p, linkage = %d\n", sc->scopesym, sc->linkage);
 
-#if 0
-    // Add import of "object" if this module isn't "object"
-    if (ident != Id::object)
-    {
-        Import *im = new Import(0, NULL, Id::object, NULL, 0);
-        members->shift(im);
-    }
-
-    // Add all symbols into module's symbol table
-    symtab = new DsymbolTable();
-    for (size_t i = 0; i < members->dim; i++)
-    {
-        Dsymbol *s = (Dsymbol *)members->data[i];
-        s->addMember(NULL, sc->scopesym, 1);
-    }
-
-    /* Set scope for the symbols so that if we forward reference
-     * a symbol, it can possibly be resolved on the spot.
-     * If this works out well, it can be extended to all modules
-     * before any semantic() on any of them.
-     */
-    for (size_t i = 0; i < members->dim; i++)
-    {
-        Dsymbol *s = (Dsymbol *)members->data[i];
-        s->setScope(sc);
-    }
-#endif
-
     // Pass 1 semantic routines: do public side of the definition
     for (size_t i = 0; i < members->dim; i++)
     {
@@ -832,6 +805,11 @@ void Module::semantic()
         runDeferredSemantic();
     }
 
+    if (userAttribDecl)
+    {
+        userAttribDecl->semantic(sc);
+    }
+
     if (!scope)
     {
         sc = sc->pop();
@@ -843,16 +821,6 @@ void Module::semantic()
 
 void Module::semantic2()
 {
-    if (deferred.dim)
-    {
-        for (size_t i = 0; i < deferred.dim; i++)
-        {
-            Dsymbol *sd = deferred[i];
-
-            sd->error("unable to resolve forward reference in definition");
-        }
-        return;
-    }
     //printf("Module::semantic2('%s'): parent = %p\n", toChars(), parent);
     if (semanticRun != PASSsemanticdone)       // semantic() not completed yet - could be recursive call
         return;
@@ -871,6 +839,11 @@ void Module::semantic2()
         s->semantic2(sc);
     }
 
+    if (userAttribDecl)
+    {
+        userAttribDecl->semantic2(sc);
+    }
+
     sc = sc->pop();
     sc->pop();
     semanticRun = PASSsemantic2done;
@@ -898,60 +871,16 @@ void Module::semantic3()
         s->semantic3(sc);
     }
 
+    if (userAttribDecl)
+    {
+        userAttribDecl->semantic3(sc);
+    }
+
     sc = sc->pop();
     sc->pop();
     semanticRun = PASSsemantic3done;
 }
 
-void Module::inlineScan()
-{
-    if (semanticRun != PASSsemantic3done)
-        return;
-    semanticRun = PASSinline;
-
-    // Note that modules get their own scope, from scratch.
-    // This is so regardless of where in the syntax a module
-    // gets imported, it is unaffected by context.
-    //printf("Module = %p\n", sc.scopesym);
-
-    for (size_t i = 0; i < members->dim; i++)
-    {
-        Dsymbol *s = (*members)[i];
-        //if (global.params.verbose)
-        //    fprintf(global.stdmsg, "inline scan symbol %s\n", s->toChars());
-        s->inlineScan();
-    }
-    semanticRun = PASSinlinedone;
-}
-
-/****************************************************
- */
-
-#if IN_DMD
-void Module::gensymfile()
-{
-    OutBuffer buf;
-    HdrGenState hgs;
-
-    //printf("Module::gensymfile()\n");
-
-    buf.printf("// Sym file generated from '%s'", srcfile->toChars());
-    buf.writenl();
-
-    for (size_t i = 0; i < members->dim; i++)
-    {
-        Dsymbol *s = (*members)[i];
-        s->toCBuffer(&buf, &hgs);
-    }
-
-    // Transfer image to file
-    symfile->setbuffer(buf.data, buf.offset);
-    buf.data = NULL;
-
-    writeFile(loc, symfile);
-}
-#endif
-
 /**********************************
  * Determine if we need to generate an instance of ModuleInfo
  * for this Module.
@@ -971,18 +900,47 @@ Dsymbol *Module::search(Loc loc, Identifier *ident, int flags)
      */
 
     //printf("%s Module::search('%s', flags = %d) insearch = %d\n", toChars(), ident->toChars(), flags, insearch);
-    Dsymbol *s;
     if (insearch)
-        s = NULL;
-    else
+        return NULL;
+    if (searchCacheIdent == ident && searchCacheFlags == flags)
     {
-        insearch = 1;
-        s = ScopeDsymbol::search(loc, ident, flags);
-        insearch = 0;
+        //printf("%s Module::search('%s', flags = %d) insearch = %d searchCacheSymbol = %s\n",
+        //        toChars(), ident->toChars(), flags, insearch, searchCacheSymbol ? searchCacheSymbol->toChars() : "null");
+        return searchCacheSymbol;
+    }
+
+    unsigned int errors = global.errors;
+
+    insearch = 1;
+    Dsymbol *s = ScopeDsymbol::search(loc, ident, flags);
+    insearch = 0;
+
+    if (errors == global.errors)
+    {
+        // Bugzilla 10752: We can cache the result only when it does not cause
+        // access error so the side-effect should be reproduced in later search.
+        searchCacheIdent = ident;
+        searchCacheSymbol = s;
+        searchCacheFlags = flags;
     }
     return s;
 }
 
+Dsymbol *Module::symtabInsert(Dsymbol *s)
+{
+    searchCacheIdent = NULL;       // symbol is inserted, so invalidate cache
+    return Package::symtabInsert(s);
+}
+
+void Module::clearCache()
+{
+    for (size_t i = 0; i < amodules.dim; i++)
+    {
+        Module *m = amodules[i];
+        m->searchCacheIdent = NULL;
+    }
+}
+
 /*******************************************
  * Can't run semantic on s now, try again later.
  */
@@ -1117,42 +1075,63 @@ int Module::imports(Module *m)
 }
 
 /*************************************
- * Return !=0 if module imports itself.
+ * Return true if module imports itself.
  */
 
-int Module::selfImports()
+bool Module::selfImports()
 {
     //printf("Module::selfImports() %s\n", toChars());
-    if (!selfimports)
+    if (selfimports == 0)
     {
         for (size_t i = 0; i < amodules.dim; i++)
-        {
-            Module *mi = amodules[i];
-            //printf("\t[%d] %s\n", i, mi->toChars());
-            mi->insearch = 0;
-        }
+            amodules[i]->insearch = 0;
 
         selfimports = imports(this) + 1;
 
         for (size_t i = 0; i < amodules.dim; i++)
-        {
-            Module *mi = amodules[i];
-            //printf("\t[%d] %s\n", i, mi->toChars());
-            mi->insearch = 0;
-        }
+            amodules[i]->insearch = 0;
     }
-    return selfimports - 1;
+    return selfimports == 2;
 }
 
+/*************************************
+ * Return true if module imports root module.
+ */
+
+bool Module::rootImports()
+{
+    //printf("Module::rootImports() %s\n", toChars());
+    if (rootimports == 0)
+    {
+        for (size_t i = 0; i < amodules.dim; i++)
+            amodules[i]->insearch = 0;
+
+        rootimports = 1;
+        for (size_t i = 0; i < amodules.dim; ++i)
+        {
+            Module *m = amodules[i];
+            if (m->isRoot() && imports(m))
+            {
+                rootimports = 2;
+                break;
+            }
+        }
+
+        for (size_t i = 0; i < amodules.dim; i++)
+            amodules[i]->insearch = 0;
+    }
+    return rootimports == 2;
+}
 
 /* =========================== ModuleDeclaration ===================== */
 
-ModuleDeclaration::ModuleDeclaration(Loc loc, Identifiers *packages, Identifier *id, bool safe)
+ModuleDeclaration::ModuleDeclaration(Loc loc, Identifiers *packages, Identifier *id)
 {
     this->loc = loc;
     this->packages = packages;
     this->id = id;
-    this->safe = safe;
+    this->isdeprecated = false;
+    this->msg = NULL;
 }
 
 char *ModuleDeclaration::toChars()
@@ -1169,8 +1148,7 @@ char *ModuleDeclaration::toChars()
         }
     }
     buf.writestring(id->toChars());
-    buf.writeByte(0);
-    return (char *)buf.extractData();
+    return buf.extractString();
 }
 
 /* =========================== Package ===================== */
@@ -1188,6 +1166,44 @@ const char *Package::kind()
     return "package";
 }
 
+Module *Package::isPackageMod()
+{
+    if (isPkgMod == PKGmodule)
+    {
+        return mod;
+    }
+    return NULL;
+}
+
+/**
+ * Checks if pkg is a sub-package of this
+ *
+ * For example, if this qualifies to 'a1.a2' and pkg - to 'a1.a2.a3',
+ * this function returns 'true'. If it is other way around or qualified
+ * package paths conflict function returns 'false'.
+ *
+ * Params:
+ *  pkg = possible subpackage
+ *
+ * Returns:
+ *  see description
+ */
+bool Package::isAncestorPackageOf(Package* pkg)
+{
+    while (pkg)
+    {
+        if (this == pkg)
+            return true;
+
+        if (!pkg->parent)
+            break;
+
+        pkg = pkg->parent->isPackage();
+    }
+
+    return false;
+}
+
 /****************************************************
  * Input:
  *      packages[]      the pkg1.pkg2 of pkg1.pkg2.mod
@@ -1340,5 +1356,3 @@ const char *lookForSourceFile(const char *filename)
     }
     return NULL;
 }
-
-
