@@ -20,11 +20,13 @@ enum TOK;
 class DValue;
 struct Loc;
 class Type;
-namespace llvm { class Value; }
+namespace llvm {
+class Value;
+}
 
-DValue* DtoAAIndex(Loc& loc, Type* type, DValue* aa, DValue* key, bool lvalue);
-DValue* DtoAAIn(Loc& loc, Type* type, DValue* aa, DValue* key);
-DValue* DtoAARemove(Loc& loc, DValue* aa, DValue* key);
-llvm::Value* DtoAAEquals(Loc& loc, TOK op, DValue* l, DValue* r);
+DValue *DtoAAIndex(Loc &loc, Type *type, DValue *aa, DValue *key, bool lvalue);
+DValue *DtoAAIn(Loc &loc, Type *type, DValue *aa, DValue *key);
+DValue *DtoAARemove(Loc &loc, DValue *aa, DValue *key);
+llvm::Value *DtoAAEquals(Loc &loc, TOK op, DValue *l, DValue *r);
 
 #endif // LDC_GEN_AA_H
