@@ -15,10 +15,8 @@
 #ifndef __LDC_GEN_ABI_WIN64_H__
 #define __LDC_GEN_ABI_WIN64_H__
 
-#include "gen/abi.h"
-
-
-TargetABI* getWin64TargetABI();
+struct TargetABI;
 
+TargetABI *getWin64TargetABI();
 
 #endif
