@@ -25,16 +25,18 @@
 ///////////////////////////////////////////////////////////
 
 /// Creates the context value for a nested function.
-void DtoCreateNestedContext(FuncDeclaration* fd);
+void DtoCreateNestedContext(FuncDeclaration *fd);
 
 /// Resolves the nested context for classes and structs with arbitrary nesting.
-void DtoResolveNestedContext(Loc& loc, AggregateDeclaration *decl, LLValue *value);
+void DtoResolveNestedContext(Loc &loc, AggregateDeclaration *decl,
+                             LLValue *value);
 
 /// Gets the context value for a call to a nested function or creating a nested
 /// class or struct with arbitrary nesting.
-llvm::Value* DtoNestedContext(Loc& loc, Dsymbol* sym);
+llvm::Value *DtoNestedContext(Loc &loc, Dsymbol *sym);
 
 /// Gets the DValue of a nested variable with arbitrary nesting.
-DValue* DtoNestedVariable(Loc& loc, Type* astype, VarDeclaration* vd, bool byref = false);
+DValue *DtoNestedVariable(Loc &loc, Type *astype, VarDeclaration *vd,
+                          bool byref = false);
 
 #endif
