@@ -18,29 +18,33 @@
 // For llvm::CodeGenOpt::Level
 #include "llvm/Support/CodeGen.h"
 
-#if LDC_LLVM_VER >= 303
 #include "llvm/Support/CommandLine.h"
 
 namespace opts {
 
-enum SanitizerCheck { None, AddressSanitizer, MemorySanitizer, ThreadSanitizer };
+enum SanitizerCheck {
+  None,
+  AddressSanitizer,
+  MemorySanitizer,
+  ThreadSanitizer
+};
 
 extern llvm::cl::opt<SanitizerCheck> sanitize;
 }
-#endif
 
-namespace llvm { class Module; }
+namespace llvm {
+class Module;
+}
 
-bool ldc_optimize_module(llvm::Module* m);
+bool ldc_optimize_module(llvm::Module *m);
 
 // Returns whether the normal, full inlining pass will be run.
 bool willInline();
 
-bool optimize();
+bool isOptimizationEnabled();
 
 llvm::CodeGenOpt::Level codeGenOptLevel();
 
-void verifyModule(llvm::Module* m);
+void verifyModule(llvm::Module *m);
 
 #endif
-
