@@ -14,25 +14,22 @@
 #ifndef LDC_GEN_RUNTIME_H
 #define LDC_GEN_RUNTIME_H
 
-namespace llvm
-{
-    class Function;
-    class GlobalVariable;
-    class Module;
+namespace llvm {
+class Function;
+class GlobalVariable;
+class Module;
 }
 
 struct Loc;
 
 // D runtime support helpers
+bool initRuntime();
+void freeRuntime();
 
-bool LLVM_D_InitRuntime();
-void LLVM_D_FreeRuntime();
+llvm::Function *getRuntimeFunction(const Loc &loc, llvm::Module &target,
+                                          const char *name);
 
-llvm::Function* LLVM_D_GetRuntimeFunction(const Loc &loc, llvm::Module* target, const char* name);
-
-llvm::GlobalVariable* LLVM_D_GetRuntimeGlobal(const Loc &loc, llvm::Module* target, const char* name);
-
-#define _adEq "_adEq2"
-#define _adCmp "_adCmp2"
+llvm::GlobalVariable *
+getRuntimeGlobal(const Loc &loc, llvm::Module &target, const char *name);
 
 #endif // LDC_GEN_RUNTIME_H
